/* 
 * Copyright (c) 2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <cmath>
#include <SDL_opengl.h>
#include "screen.h"
#include "camera.h"


Camera::Camera (Screen *screen)
{
    _screen = screen;
    resize ();

    glEnable (GL_TEXTURE_2D);
	glShadeModel (GL_SMOOTH);
	glClearColor (0,0,0,0);
    glClearDepth (1.0f);
    glEnable (GL_DEPTH_TEST);
    glDepthFunc (GL_LEQUAL);
    glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glDisable (GL_LIGHTING);

    _screen->set_camera (this);
    resize ();
}

Camera::~Camera (void)
{}

void
Camera::resize (void)
{
	GLfloat ratio;
    int height = _screen->height();
	int width  = _screen->width();
	if (height == 0)
		height = 1;

    int x = 0;
    int y = 0;
    if (_screen->flags() & SDL_FULLSCREEN) {
        x = (_screen->fullwidth() - _screen->width())/2;
        y = (_screen->fullheight() - _screen->height())/2;
    }

	ratio = width/float(height);
	glViewport (x, y, (GLint) width, (GLint) height);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (30.0f, ratio, 1.0f, 100.0f);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
}

void
Camera::look (void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity ();

	gluLookAt (3, 3, 3,
			   0, 0, 0,
			   0, 1, 0);
}

void
Camera::look_at (float eyeX, float eyeY, float eyeZ, 
                 float centerX, float centerY, float centerZ,
                 float upX, float upY, float upZ)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity ();

	gluLookAt (eyeX, eyeY, eyeZ,
               centerX, centerY, centerZ,
               upX, upY, upZ);
}
