/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit website at www.pixelpracht.net
// ================================================================================

#include "log.h"
#include "texture.h"
#include "fx_effect.h"
#include "fx_cloud.h"

using namespace fx;


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Burn::init (void)
{
	TextureManager *tmanager = TextureManager::it();
	last_emission_ = 0.1f;
	emission_rate_ = 0.1f;

	Cloud *c = new Cloud (this, 1000);
	c->type()           = FX_SPRITE;
	c->blend()          = FX_TRANSPARENT;
	c->texture()        = tmanager->get (path_+"smoke-ring.png", true);
	c->fade_in_stop()   = 0.0f;
	c->fade_out_start() = 0.3f;
	clouds_["Smoke"] = c;

	c = new Cloud (this, 300);
	c->type()           = FX_TEX_QUAD;
	c->blend()          = FX_LIGHT;
	c->texture()        = tmanager->get (path_+"star.jpg");
	c->fade_in_stop()   = 0.3f;
	c->fade_out_start() = 0.8f;
	clouds_["Shine"] = c;

	c = new Cloud (this, 2000);
	c->type()           = FX_SPRITE;
	c->blend()          = FX_LIGHT;
	c->texture()        = tmanager->get (path_+"sparkooo.png", true);
	c->fade_in_stop()   = 0.0f;
	c->fade_out_start() = 0.7f;
	clouds_["Spark"] = c;

	c = new Cloud (this, 300);
	c->type()           = FX_SPRITE;
	c->blend()          = FX_GLOW;
	c->texture()        = tmanager->get (path_+"flame.png", true);
	c->fade_in_stop()   = 0.3f;
	c->fade_out_start() = 0.7f;
	clouds_["Burn"] = c;


	templates_["SmokeMin"].lifespan() = 3.0f;
	templates_["SmokeMin"].size()     = 0.08f;
	templates_["SmokeMin"].sat()      = 0.2f;
	templates_["SmokeMin"].density()  = 3.0f;
	templates_["SmokeMin"].velocity() = vector (0.1f, 0.3f, 0.7f);

	templates_["SmokeMax"].lifespan() = 1.5f;
	templates_["SmokeMax"].size()     = 0.12f;
	templates_["SmokeMax"].sat()      = 0.5f;
	templates_["SmokeMax"].density()  = 1.0f;
	templates_["SmokeMax"].velocity() = vector (0.2f, 0.9f, 0.2f);

	templates_["ShineMin"].lifespan() = 0.4f;
	templates_["ShineMin"].size()     = 0.3f;
	templates_["ShineMin"].sat()      = 0.2f;
	templates_["ShineMin"].color()    = vector (0.7f, 0.0f, 0.0f);
	templates_["ShineMin"].position() = vector (-0.07f, -0.15f, -0.07f);

	templates_["ShineMax"].lifespan() = 0.6f;
	templates_["ShineMax"].size()     = 0.9f;
	templates_["ShineMax"].sat()      = 0.4f;
	templates_["ShineMax"].color()    = vector (1.0f, 0.2f, 0.8f);
	templates_["ShineMax"].position() = vector (0.07f, -0.05f, 0.07f);

	templates_["BurnMin"].lifespan() = 0.5f;
	templates_["BurnMin"].size()     = 0.05f;
	templates_["BurnMin"].sat()      = 0.2f;
	templates_["BurnMin"].spin()     = 360.0f;
	templates_["BurnMin"].position() = vector (0.0f, -0.01f, 0.0f);

	templates_["BurnMax"].lifespan() = 0.6f;
	templates_["BurnMax"].size()     = 0.25f;
	templates_["BurnMax"].sat()      = 0.6f;
	templates_["BurnMax"].spin()     = -180.0f;
	templates_["BurnMax"].position() = vector (0.0f, 0.01f, 0.0f);

	templates_["SparkMin"].lifespan() = 0.8f;
	templates_["SparkMin"].size()     = 0.03f;
	templates_["SparkMin"].sat()      = 0.7f;
	templates_["SparkMin"].density()  = 5.0f;
	templates_["SparkMin"].velocity() = vector (-1.0f, 0.1f, -1.0f);
	templates_["SparkMin"].color()    = vector (0.8f, 0.8f, 0.8f);

	templates_["SparkMax"].lifespan() = 1.0f;
	templates_["SparkMax"].size()     = 0.1f;
	templates_["SparkMax"].sat()      = 1.0f;
	templates_["SparkMax"].density()  = 40.0f;
	templates_["SparkMax"].velocity() = vector (1.0f, 0.0f, 1.0f);
	templates_["SparkMax"].color()    = vector (1.0f, 1.0f, 1.0f);
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Burn::step_in_time (const float dt) {
	for (std::map<std::string, Cloud *>::iterator i=clouds_.begin(); i != clouds_.end(); i++)
		i->second->step_in_time (dt);
	last_emission_ += dt;

	while (last_emission_ > emission_rate_) {
		Particle p1(templates_["SmokeMin"], templates_["SmokeMax"]);
		clouds_["Smoke"]->add (p1);

		Particle p2(templates_["ShineMin"], templates_["ShineMax"]);
		clouds_["Shine"]->add (p2);

		Particle p3(templates_["SparkMin"], templates_["SparkMax"]);
		clouds_["Spark"]->add (p3);

		Particle p4(templates_["BurnMin"], templates_["BurnMax"]);
		clouds_["Burn"]->add (p4);
		last_emission_ -= emission_rate_;
	}
	clouds_["Smoke"]->flow (0.0f, 0.3f, 0.1f, 0.3f, dt);
	clouds_["Smoke"]->linear_gravity (0.0f, -1.0f, 0.0f, 0.2f, dt);
	clouds_["Smoke"]->scale  (SIZE, 1.0f+dt*0.7f);
	clouds_["Spark"]->linear_gravity (0.0f, -1.0f, 0.0f, 2.5f, dt);
}
