/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit website at www.pixelpracht.net
// ================================================================================

#ifndef	_CLOUD_H
#define	_CLOUD_H

#include <vector>
#include "texture.h"
#include "fx_struct.h"
#include "fx_effect.h"
#include "fx_particle.h"


namespace fx {
	class Cloud {

		// ===================================================================
		// - Public methods --------------------------------------------------
		// ===================================================================
	public:
		Cloud (class Effect *effect, unsigned int n);
		~Cloud (void);
		void add (Particle &p, unsigned int n=1);
		void kill (int index = -1);

		// Manipulation
		void scale (const unsigned mask, const vector &v);
		void scale (const unsigned mask, const float scale);
		void scale (const unsigned mask, const float scale_x, const float scale_y, const float scale_z);
		// Physics
		void step_in_time (const float dt);
		void bullet (const vector &target);
		void bullet (const float tx, const float ty, const float tz);
		void linear_gravity (const vector &point, const float gravity, const float dt);
		void linear_gravity (const float px, const float py, const float pz, const float gravity, const float dt);
		void circular_gravity (const vector &point, const float gravity, const float dt);
		void circular_gravity (const float px, const float py, const float pz, const float gravity, const float dt);
		void point_gravity (const vector &point, const float radius, const float mass, const float dt);
		void point_gravity (const float px, const float py, const float pz, const float radius, const float mass, const float dt);
		void friction (const float viscosity, const float dt);
		void flow (const vector &direction, const float viscosity, const float dt);
		void flow (const float dx, const float dy, const float dz, const float viscosity, const float dt);
		// Render
		void render (void);


		// ===================================================================
		// - Private features ------------------------------------------------
		// ===================================================================
	private:
		class Effect *					effect_;		// System this cloud belongs to
		std::vector<class Particle>		particles_;		// Particles for this cloud (size fixed at creation)
		int		 						first_dead_;	// Index of first dead particle 
		e_type							type_;			// Type of particles (same for all then)
		e_blend							blend_;			// Blend mode of particles
		Texture *						texture_;		// Texture to use (if necessary, depends on type)
		tex_tile						tile_;			// Texture tile (uv mapping)
		float							elongation_;	// Elongation
		float							fade_in_stop_;	// Fade in stop (relative to particle lifespan)
		float							fade_out_start_;// Fade out start (relative to particle lifespan)
		bool							zsort_;			// Do we need z sorting ?


		// ===================================================================
		// - Features manipulation--------------------------------------------
		// ===================================================================
	public:
		class Effect *					&effect (void)			{return effect_;}
		std::vector<class Particle>		&particles (void)		{return particles_;}
		Particle						&particles (int i)		{return particles_.at(i);}
		Particle						&operator [](int i)		{return particles_.at(i);}
		int		 						&first_dead (void)		{return first_dead_;}
		e_type							&type (void)			{return type_;}
		e_blend							&blend (void)			{return blend_;}
		Texture *						&texture (void)			{return texture_;}
		tex_tile						&tile (void)			{return	tile_;}
		float							&elongation (void)		{return elongation_;}
		float							&fade_in_stop (void)	{return fade_in_stop_;}
		float							&fade_out_start (void)	{return fade_out_start_;}
		bool							&zsort (void)			{return zsort_;}
	};
}

#endif // _CLOUD_H
