/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit his website at www.pixelpracht.net
// ================================================================================

#include <functional>
#include "fx_effect.h"

using namespace fx;


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
bool particle_comparison (Particle *p1, Particle *p2) {return p1->z()<p2->z();}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
Effect::Effect (std::string path)
{
	age_ = 0;
	path_ = path;
	if (path.empty())
		path = "./";
	if (path_[path_.size()-1] != '/')
		path_ += "/";
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
Effect::~Effect (void)
{
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Effect::add (Particle *p)
{
	sorted_.remove (p);
	sorted_.push_back (p);
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Effect::remove (Particle *p)
{
	sorted_.remove (p);
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Effect::update_z (void)
{
	float m[16];
	glGetFloatv (GL_MODELVIEW_MATRIX, m);

	for (std::list<Particle *>::iterator i=sorted_.begin(); i != sorted_.end(); i++)
		(*i)->z((*i)->position().x*m[2]
				+ (*i)->position().y*m[6]
				+ (*i)->position().z*m[10]
				+ m[14]);
	sorted_.sort (particle_comparison);
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Effect::render (void)
{
	glDepthMask (false);
	glPushAttrib (GL_ALL_ATTRIB_BITS);

	// Rendering unsorted particles
 	for (std::map<std::string, Cloud *>::iterator i=clouds_.begin(); i != clouds_.end(); i++) {
		Cloud *c = i->second;
		if (!c->zsort()) {
			c->render ();
			for (unsigned int j=0; j<c->particles().size(); j++)
				c->particles(j).render (1.0f);
		}
    }

	// Rendering sorted particles
	update_z();
	Cloud *cloud = 0;
	for (std::list<Particle *>::iterator i=sorted_.begin(); i != sorted_.end(); i++) {
		if ((*i)->cloud() != cloud) {
			(*i)->cloud()->render();
			cloud = (*i)->cloud();
		}
		(*i)->render (1.0f);
	}

	glPopAttrib ();
	glDepthMask (true);
}


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Effect::reset (void)
{
	age_ = 0;
 	for (std::map<std::string, Cloud *>::iterator i=clouds_.begin(); i != clouds_.end(); i++) {
		i->second->kill();
    }
	sorted_.clear();
}
