/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit website at www.pixelpracht.net
// ================================================================================

#ifndef	_EFFECT_H
#define	_EFFECT_H

#include <string>
#include <list>
#include <map>
#include <SDL/SDL_mixer.h>
#include "fx_struct.h"
#include "fx_cloud.h"
#include "fx_particle.h"


namespace fx {

	class Effect {

		// ===================================================================
		// - Public methods --------------------------------------------------
		// ===================================================================
	public:
		Effect (std::string path = "./pics/");
		virtual ~Effect (void);
		void add (class Particle *p);
		void remove (class Particle *p);
		void update_z (void);
		void render (void);
		virtual void init (void) = 0;
		virtual void reset (void);
		virtual void step_in_time (const float dt) = 0;

		// ===================================================================
		// - Private features ------------------------------------------------
		// ===================================================================
	protected:
		float									age_;
		float									last_emission_;
		float 									emission_rate_;
		Mix_Chunk *								sample_;
		int										channel_;
		std::string 							path_;
		std::list<class Particle *>				sorted_;
		std::map<std::string, class Particle>	templates_;
		std::map<std::string, class Cloud *>	clouds_;

		// ===================================================================
		// - Features manipulation--------------------------------------------
		// ===================================================================
	public:
		std::list<class Particle *> &sorted (void)		{return sorted_;}
		Particle &templates (std::string name)			{return templates_[name];}
		Cloud *clouds (std::string name)				{return clouds_[name];}
	};


	
	class Explosion : public Effect {
		// ===================================================================
		// - Public methods --------------------------------------------------
		// ===================================================================
	public:
		Explosion (std::string path = "./pics/") : Effect (path) {};
		void init (void);
		void step_in_time (const float dt);
	};


	
	class Burn : public Effect {
		// ===================================================================
		// - Public methods --------------------------------------------------
		// ===================================================================
	public:
		Burn (std::string path = "./pics/") : Effect (path) {};
		void init (void);
		void step_in_time (const float dt);
	};
}

#endif // _EFFECT_H
