/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit website at www.pixelpracht.net
// ================================================================================

#ifndef	_PARTICLE_H
#define	_PARTICLE_H

#include "fx_struct.h"


namespace fx {

	class  Particle {

		// ===================================================================
		// - Public methods --------------------------------------------------
		// ===================================================================
	public:
		Particle (void);
		Particle (const Particle &p1, const Particle &p2);
		Particle (const Particle &p1, const Particle &p2, const float factor);
		Particle &operator = (const Particle &other);
		float z (void) const {return z_;}

		// Manipulation
		void scale (const unsigned mask, const vector &v);
		void scale (const unsigned mask, const float scale);
		void scale (const unsigned mask, const float scale_x, const float scale_y, const float scale_z);
		void add (const unsigned mask, const Particle &other);
		void sub (const unsigned mask, const Particle &other);
		void add (const unsigned mask, const Particle &other, int sign);
		void set (const unsigned mask, const Particle &other);
		void neg (const unsigned mask);
		void interpolate (const unsigned mask, const Particle &p1, const Particle &p2, const float factor);
		void fit_sphere (const vector &center, const float radius);
		void fit_sphere (const float cx, const float cy, const float cz, const float radius);
		void fit_cube (const vector &center, const float size);
		void fit_cube (const float cx, const float cy, const float cz, const float size);

		// Physics
		void bullet (const vector &target);
		void bullet (const float tx, const float ty, const float tz);
		void linear_gravity (const vector &direction, const float gravity, const float dt);
		void linear_gravity (const float dx, const float dy, const float dz, const float gravity, const float dt);
		void circular_gravity (const vector &point, const float gravity, const float dt);
		void circular_gravity (const float px, const float py, const float pz, const float gravity, const float dt);
		void point_gravity (const vector &point, const float radius, const float mass, const float dt);
		void point_gravity (const float px, const float py, const float pz, const float radius, const float mass, const float dt);
		void friction (const float viscosity, const float dt);
		void flow (const vector &direction, const float viscosity, const float dt);
		void flow (const float dx, const float dy, const float dz, const float viscosity, const float dt);

		// Render
		float fade (void);
		void billboard (bool mode);
		void render (const float alpha);


		// ===================================================================
		// - Private features ------------------------------------------------
		// ===================================================================
	private:
		vector			position_;	// position
		vector			velocity_;	// velocity
		float			density_;	// density
		float			mass_;		// mass
		float			size_;		// size
		float			spin_;		// degree/s, no effects for sparks and points
		float			rotation_;	// current rotation in degree
		vector			color_;		// color
		float			sat_;		// ??
		float			lifespan_;	// lifespan (in seconds)
		float			age_;		// age (in seconds)
		bool			dead_;		// dead flag
		float			z_;			// z for z depth sorting
		class Cloud * 	cloud_;		// Cliud of particle this particule belongs to


		// ===================================================================
		// - Features manipulation--------------------------------------------
		// ===================================================================
	public:
		vector 		 		& position (void)	{return position_;}
		vector 		 		& velocity (void)	{return velocity_;}
		float  	 			& density (void)	{return density_;}
		float 		 		& mass (void)		{return mass_;}
		float 		 		& size (void)		{return size_;}
		float  	 			& spin (void)		{return spin_;}
		float  	 			& rotation (void)	{return rotation_;}
		vector  			& color (void)		{return color_;}
		float  		 		& sat (void)		{return sat_;}
		float  		 		& lifespan (void)	{return lifespan_;}
		float  		 		& age (void)		{return age_;}
		bool  		 		& dead (void)		{return dead_;}
		void  		 		z (float new_z)		{z_ = new_z;}
		class Cloud * 		& cloud (void)		{return cloud_;}
	};
}

#endif // _PARTICLE_H
