/* 
 * Copyright (c) 2004 Thomas Jahn & Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ================================================================================
// Notes
// --------------------------------------------------------------------------------
// This code is adapted from "PixelprachtFX Demo" by Thomas Jahn aka Lithander.
// For more stuff, please visit website at www.pixelpracht.net
// ================================================================================

#ifndef	_FX_STRUCT_H
#define	_FX_STRUCT_H

#include <SDL/SDL_opengl.h>


namespace fx {

	// ================================================================================
	// vector structure
	// --------------------------------------------------------------------------------
	// vector structure uses union so that fields (vector components) can
	// use different names
	// ================================================================================
	struct vector {
		union {
			float x;			// x component
			float dx;			//  or dx
			float r;			//  or r
			float red;			//  or red
		};
		union {
			float y;			// y component
			float dy;			//  or dy
			float g;			//  or g
			float green;		//  or green
		};
		union {
			float z;			// z component
			float dz;			//  or dz
			float b;			//  or b
			float blue;			//  or blue
		};
		// Default initialization
		vector (void) {
			x = 0;
			y = 0;
			z = 0;
		}
		vector (const float a, const float b, const float c) {
			x = a;
			y = b;
			z = c;
		}
	};

	// ================================================================================
	// texture tile structure
	// --------------------------------------------------------------------------------
	//
	// ================================================================================
	struct tex_tile {
		float min_u;
		float min_v;
		float max_u;
		float max_v;
		tex_tile (void) {
			min_u = min_v = max_u = max_v = 0.0f;
		}

	};

	// ================================================================================
	// Constants
	// --------------------------------------------------------------------------------
	// 
	// ================================================================================
#   define GRAV_MIN_RADIUS		0.2;
#   define ON					true
#   define OFF					false

	// ================================================================================
	// Particle attribute bitmasks
	// --------------------------------------------------------------------------------
	// 
	// ================================================================================
#   define POSITION	0x0001
#   define VELOCITY	0x0002
#   define COLOR	0x0004
#   define MASS		0x0008
#   define SIZE		0x0010
#   define DENSITY	0x0020
#   define SPIN		0x0040
#   define ROTATION	0x0080
#   define SAT		0x0100
#   define LIFESPAN	0x0200

#   define POS_X	0x0400
#   define POS_Y	0x0800
#   define POS_Z	0x1000
#   define VEL_X	0x2000
#   define VEL_Y	0x4000
#   define VEL_Z	0x8000

#   define ALL		0x03FF

	// ================================================================================
	// particle types
	// --------------------------------------------------------------------------------
	// 
	// ================================================================================
	typedef enum {
		FX_POINT,			//
		FX_QUAD,			//
		FX_TEX_QUAD,		//
		FX_SPARK,			//
		FX_TEX_SPARK,		//
		FX_SPRITE			//
	} e_type;

	// ================================================================================
	// blend modes
	// --------------------------------------------------------------------------------
	// 
	// ================================================================================
	typedef enum {
		FX_OPAQUE,			//
		FX_GLOW,			//
		FX_TRANSPARENT,		//
		FX_LIGHT			//
	} e_blend;
}

#endif // _FX_H
