/* 
 * Copyright (c) 2005 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef	__SCREEN_H__
#define	__SCREEN_H__

#include <vector>
#include <string>
#include <SDL/SDL.h>


class Screen {
	// ===================================================================
	// - Public methods --------------------------------------------------
	// ===================================================================
 public:
	Screen (int width=800, int height=600, Uint32 bpp=24);
	~Screen (void);

	void attach (class Camera *camera);
	void resize (int width, int height);
	void update (void);
	void full (void);

	void blit_surface (SDL_Surface *src, SDL_Rect &srcrect, SDL_Rect &dstrect, float alpha=1.0);
	int screenshot (void);
	int export_png (SDL_Surface *src, std::string filename);
    

	// ===================================================================
	// - Private features ------------------------------------------------
	// ===================================================================
 private:
	int				_width;	
	int				_height;
    int 			_fullwidth;
    int				_fullheight;

	Uint8			_bpp;
    Uint32			_flags;
    class Camera	*_camera;
	SDL_Surface *	_surface;

	// ===================================================================
	// - Features manipulation -------------------------------------------
	// ===================================================================
 public:
	SDL_Surface *	surface (void)		{return _surface;}
    int				width (void)		{return _width;}
	int				height (void)		{return _height;}
	int				fullwidth (void)	{return _fullwidth;}
	int				fullheight (void)	{return _fullheight;}
    Uint32			flags (void)		{return _flags;}

    void			set_camera (Camera *camera)	{_camera = camera;}
    
};

#endif // __SCREEN_H__
