/* 
 * Copyright (c) 2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef	_TEXTURE_H
#define	_TEXTURE_H

#include <vector>
#include <string>
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include "singleton.h"



class Texture
{
    // ===================================================================
	// - Public methods --------------------------------------------------
	// ===================================================================
 public:
    Texture (std::string filename, bool alpha = false, bool mipmaps = true, bool repeat = false);
    ~Texture (void);

    void generate (void);
    std::string	&get_filename (void)	{return _filename;}
    bool		&get_alpha (void)		{return _alpha;}
    bool		&get_mipmaps (void)		{return _mipmaps;}
    bool		&get_repeat(void)		{return _repeat;}
    GLuint		&get_id (void)			{return _id;}

	// ===================================================================
	// - Private methods -------------------------------------------------
	// ===================================================================
 private:
	GLubyte *load (int &width, int &height);
	Uint32 pixel (SDL_Surface *surface, int x, int y);

	// ===================================================================
	// - Private features ------------------------------------------------
	// ===================================================================
 private:
    std::string		_filename;
    bool			_alpha;
    bool			_mipmaps;
    bool			_repeat;
    GLuint			_id;
};
    


class TextureManager : public Singleton<TextureManager> {
	friend class Singleton<TextureManager>;

	// ===================================================================
	// - Public methods --------------------------------------------------
	// ===================================================================
 public:
	void init (void);
	Texture *get (std::string fullname, bool use_alpha = false, bool use_mipmaps = true, bool use_repeat = false);


	// ===================================================================
	// - Private methods -------------------------------------------------
	// ===================================================================
 private:
	TextureManager (void) {};
	~TextureManager (void) {};


	// ===================================================================
	// - Private features ------------------------------------------------
	// ===================================================================
	std::vector<Texture *> database_;
};


#endif	// __TEXTURE_H
