/* 
 * Copyright (c) 2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "timer.h"


// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
Timer::Timer (void)
{
	last_tick_ = 0;
	pause_tick_ = 0;
	paused_ = false;
	fps_ = 60;
}

// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
Timer::~Timer (void)
{
}

// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Timer::pause (void)
{
	if (last_tick_ == 0)
		last_tick_ = SDL_GetTicks();
	pause_tick_ = SDL_GetTicks();
	paused_ = true;
}

// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
void Timer::unpause (void)
{
	last_tick_  = SDL_GetTicks() - (pause_tick_-last_tick_);
	pause_tick_ = 0;
	paused_     = false;
}

// ================================================================================
//  
// --------------------------------------------------------------------------------
//  
// ================================================================================
bool Timer::paused (void)
{
	return paused_;
}

// ================================================================================
// wait_next_tick
// --------------------------------------------------------------------------------
// return elapsed time in seconds
// ================================================================================
float Timer::wait_next_tick (void)
{
	if (paused_)
		return 0;

	if (last_tick_ == 0) {
		last_tick_ = SDL_GetTicks();
		return 0;
	}

	Uint32 wait      = (Uint32)(1000.0f/fps_);
	Uint32 new_tick  = SDL_GetTicks();

	if ((new_tick - last_tick_) < wait)
		SDL_Delay (wait - (new_tick-last_tick_));

	float dt = (SDL_GetTicks() - last_tick_)/1000.0f;
	last_tick_ = SDL_GetTicks();
	return dt;
}

